"BlogTopic - blog interface product for CMF"

from AccessControl import ClassSecurityInfo
from Acquisition import aq_parent, aq_base
from Acquisition import Implicit

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import View, ManageProperties, \
     ListFolderContents, ModifyPortalContent, AddPortalContent
from Products.CMFTopic import Topic

from DateTime import DateTime
import re

#XXX scripts?
from datetools import seqToDate, dateToSeq

#XXX perms
factory_type_information = { 'id' : 'BlogTopic',
                             'portal_type'	: 'BlogTopic',
                             'meta_type'      : 'BlogTopic',
                             'description'    : "A BlogTopic is a Topic-based weblog interface.",
                             'icon'           : 'blogtopic.gif',
                             'product'        : 'BlogTopic',
                             'factory'        : 'addBlogTopic',
                             'filter_content_types' : 0,
                             #'allowed_content_types' : ('BlogTopic', ),
                             'immediate_view' : 'blogtopic_edit_form',
                             'actions'        :
                             ( { 'id'            : 'criteria',
                                 'name'          : 'Criteria',
                                 'action'        : 'topic_criteria_form',
                                 'permissions'   : (ManageProperties,),
                                 },
                               { 'id'            : 'view',
                                 'name'          : 'View',
                                 'action'        : 'blogtopic_view',
                                 'permissions'   : (View,),
                                 },
                               { 'id'            : 'edit',
                                 'name'          : 'Edit',
                                 'action'        : 'blogtopic_edit_form',
                                 'permissions'   : (ManageProperties,),
                                 },
                               { 'id'            : 'subblogtopics',
                                 'name'          : 'SubBlogTopics',
                                 'action'        : 'blogtopic_subblogtopics_form',
                                 'permissions'   : (AddPortalContent,),
                                 }
                               )
                             }

def addBlogTopic(self, id, title='', REQUEST = None):
    "add BlogTopic instance"
    ob = BlogTopic(id)
    ob.title = str(title)
    id = self._setObject(id, ob)
    if REQUEST: 
        return self.manage_main(self, REQUEST)

class BlogTopic(Topic.Topic):
    "Display manager class for weblog items"

    meta_type = 'BlogTopic'
    security = ClassSecurityInfo()
    dateErrorSkin = 'blogtopic_dateError'
    dateField = "created"

    def __init__(self, id, **kw):
        BlogTopic.inheritedAttribute('__init__')(self, id, kw)

    # should be using the tools in BeforeTraverse.py?
    def __before_publishing_traverse__(self, obj, req):
        """
        If the traversal stack is all intable strings that don't lead to
        attrs or items, put the date represented in the request and lop the
        stack, causing the publisher to call us for the next publishing step.
        """
        stack = req['TraversalRequestNameStack']
        # do nothing unless we have only intable strs to traverse overall
        # XXX use something faster than re.match(str)?
        if stack and (len(filter(lambda x: re.match('^[0-9]+$', x), stack)) !=
                      len(stack)):
            return
        # Do nothing if there's an attr or item to publish, allows
        # things like custom views for dates. 
        # This isn't rigorous, there could still be something to publish,
        # but there isn't a mechanism to find any name that
        # BaseRequest.traverse would publish, other than copying the code.
        stack.reverse()
        if stack:
            self_base = aq_base(self)
            if hasattr(self_base, stack[0]):
                return
            try:
                dummy = self_base[stack[0]]
                return
            except:
                pass
            
        # stop looking for next obj to publish by emptying stack
        req['TraversalRequestNameStack'] = []
        try:
            req['BlogTopicDate'] = seqToDate(stack)
        except:
            req['TraversalRequestNameStack'] = [self.dateErrorSkin]

    # Differs from Topic.buildQuery in that we don't take aq_inner,
    # tracker issue 290 suggests that this could go in if patched & tested.
    # Note that ancestor query keys that differ from self's are ANDed, but
    # keys that are the same are *replaced* by the ancestor's - for example,
    # if self adds {'Subject':'foo'} and ancestor adds {'Subject':'bar'}, then
    # we search for 'foo' only.  Would be nice to AND or disallow that.
    def buildQuery( self ):
        "Construct a catalog query using our criterion objects."
        result = {}

        if self.acquireCriteria:
            try:
                parent = aq_parent(self)
                result.update( parent.buildQuery() )
            except: # oh well, can't find parent, or it isn't a Topic.
                pass

        for criterion in self.listCriteria():
            for key, value in criterion.getCriteriaItems():
                result[ key ] = value
        
        return result

    def queryCatalog(self, REQUEST={}, **kw):
        """
        Add a date criterion based on the blog date field to the query,
        and return the result of the inherited call.  If REQUEST is not
        passed or doesn't have a date, now's date is used.
        """
        # We do this here at the final query call, not buildQuery, so that
        # it's only done once.
        # Since we do this before Topic.buildQuery, criteria for the
        # same index in this or parent topics will overwrite the dict.
        date = REQUEST.get('BlogTopicDate', DateTime())
        kw.update({self.dateField: date})
        kw.update({'%s_usage' % self.dateField: 'range:max'})
        # Topic's queryCatalog builds the query and does the search
        return apply(BlogTopic.inheritedAttribute('queryCatalog'),
                     (self, REQUEST), kw)

    security.declarePublic('getPrevDate')
    def getPrevDate(self, REQUEST={}, size=5, **kw):
        """
        Return the date corresponding to the next blog page for later entries,
        or None.
        """
        date = REQUEST.get('BlogTopicDate', DateTime())
        kw.update({'sort_on':self.dateField})
        kw.update({self.dateField: date})
        kw.update({'%s_usage' % self.dateField: 'range:min'})
        result = apply(BlogTopic.inheritedAttribute('queryCatalog'),
                       (self, REQUEST), kw)

        if not result:
            return None
        if len(result) >= size:
            end = result[size-1]
        else:
            end = result[-1]
        # XXX this could cause us to skip entries if we end up in the middle of
        # several entries with the same day
        return getattr(end, self.dateField)
    
    security.declareProtected('Add portal content', 'addSubBlogTopic')
    def addSubBlogTopic( self, id ):
        "Add a new subBlogTopic."
        try:
            tool = getToolByName( self, 'portal_types' )
        except:
            # don't know why Topic does this, you need to create with
            # constructInstance
            #self._setOb( id, BlogTopic( id ) )
            raise
        else:
            bftype = tool.getTypeInfo( self )
            bftype.constructInstance( self, id )

        return self._getOb( id )
