"""
External method installation script for BlogTopic.

To use, add an external method to the root of the CMF site:

 module name: BlogTopic.Install
 function name: install

Then go to the management screen for the newly added external method
and click the 'Test' tab.

You may then remove or leave the external method.
"""

# adapted from CMFForum

from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName

from Products.BlogTopic import blogtopic_globals

from cStringIO import StringIO
import string

def installSubSkin(self, outStream, skinFolder):
    "Install skinFolder into skin selection paths after custom."
    skinstool = getToolByName(self, 'portal_skins')
    for skin in skinstool.getSkinSelections():
        path = skinstool.getSkinPath(skin)
        path = map(string.strip, string.split(path,','))
        if not skinFolder in path:
            try:
                path.insert(path.index('custom')+1, skinFolder)
            except ValueError:
                path.append(skinFolder)
            path = string.join(path, ', ')
            skinstool.addSkinSelection(skin, path)
            outStream.write(
                'BlogTopic subskin successfully installed into %s.\n' % skin)
        else:
            outStream.write(
                'BlogTopic subskin was already installed into %s.\n' % skin)

def installSkins(self, out):
    "Install skin folder into selection paths, add skin directory view."
    skinstool = getToolByName(self, 'portal_skins')

    try:
        installSubSkin(self, out, 'blogtopic')        
        out.write('BlogTopic subskin successfully installed.\n')
    except:
        out.write('BlogTopic subskin failed to install.\n')

    try:  
        addDirectoryViews(skinstool, 'skins', blogtopic_globals)
        out.write("Added BlogTopic directory views to portal_skins.\n")
    except:
        out.write(
            'Unable to add BlogTopic directory view to portal_skins.\n')

def install(self):
    "Install skins, type."
    skinstool = getToolByName(self, 'portal_skins')

    out = StringIO()
    out.write('BlogTopic installation tool.\n')

    installSkins(self, out)

    try:
        typesTool = getToolByName(self, 'portal_types')
        typesTool.manage_addTypeInformation(
            add_meta_type = 'Factory-based Type Information',
            id = 'BlogTopic', typeinfo_name = 'BlogTopic: BlogTopic')
        out.write('Installed new Type: BlogTopic.\n')
    except:
        out.write('New type BlogTopic not installed.\n')
    return out.getvalue()

